<?php
namespace Util;

use Doctrine\ORM\Query\Expr\Func;
use Util\Query;

use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Util extends \Zeedhi\Framework\Controller\Simple {
    
    protected $connection;
    protected $tipoBanco;
    protected $utilOdhen;
    protected $logPath;
    protected $instanceManager;
    protected $consumidorAPI;

    public function __construct($connection, \Odhen\API\Util\Util $utilOdhen, $instanceManager, 
                                    \Odhen\API\Service\Consumidor $consumidorAPI) {
        $this->connection = $connection;
        $this->utilOdhen = $utilOdhen;
        $this->instanceManager = $instanceManager;
        $this->logPath = $instanceManager->getParameter('LOG_PATH');
        $this->consumidorAPI = $consumidorAPI;
    }

    public function beginTransaction(){
        return $this->connection->beginTransaction();
    }
    public function commit(){
        $this->connection->commit();
    }
    public function rollBack(){
        $this->connection->rollBack();
    }
    public function connection(){
        return $this->connection;
    }
    /* Converte os valores numéricos retornados pelo banco em float */
    public function float($value) {
        return floatval(number_format($value, 2, '.', ','));
    }
    
    /* Verifica se a data recebida está no formato requerido */
    public function isValidDate($date) {
        $format = '/^[0-9]{1,2}\/[0-9]{1,2}\/[0-9]{4}$/';
        return preg_match($format, $date);
    }
    
    /* Verifica se o intervalo de tempo limitado pelas datas é válido */
    public function isValidDateTimeInterval($inicialDate, $finalDate) {
        $format = 'd/m/Y H:m:i';
        $inicial = date_create_from_format($format, $inicialDate);
        $final   = date_create_from_format($format, $finalDate);
        
        if ($final > $inicial) {
            return true;
        } else {
            return false;
        }
    }

    public function removeDecimal($value, $offset = 2){
        return floatval(bcdiv($value, 1, $offset));
    }
    
    public function subtract($a, $b){
        return floatval(bcsub($a, $b, 6));
    }
    
    public function add($a, $b){
        return floatval(bcadd($a, $b, 6));
    }
    
    public function mult($a, $b){
        return floatval(bcmul($a, $b, 6));
    }
    
    public function div($a, $b){
        return floatval(bcdiv($a, $b, 6));
    }

    public function getPaymentId($paymentCode){
        $params = array(
            'CDTIPORECE' => $paymentCode
        );
        $paymentId = $this->connection->fetchAssoc(Query::GET_PAYMENT_ID, $params);

        return $paymentId['IDTIPORECE'];
    }
    
    public function getPosDate($subsidiaryCode, $sellerPos){
        $params = array(
            'CDFILIAL' => $subsidiaryCode,
            'CDCAIXA'  => $sellerPos
        );
        
        $result = $this->connection->fetchAssoc(Query::GET_DTABERCAIX, $params);
        
        return $result['DTABERCAIX'];
    }
    
    public function getPaymentValue($subsidiaryCode, $vendarestSeq){
        $params = array(
            'CDFILIAL' => $subsidiaryCode,
            'NRVENDAREST' => $vendarestSeq
        );
        
        $result = $this->connection->fetchAssoc(Query::GET_PAYMENT_VALUE, $params);
        
        return $result['VRMOVIVENDDLV'];
        
    }

    public function isValidDateInterval($inicialDate, $finalDate) {
        $format = 'd/m/Y';
        $inicial = date_create_from_format($format, $inicialDate);
        $final   = date_create_from_format($format, $finalDate);
        
        if ($final > $inicial) {
            return true;
        } else {
            return false;
        }
    }
    
    /* Verifica se uma filial existe */
    public function subsidiaryExists($subsidiaryCode) {
        $params = array(
            ':CDFILIAL' => $subsidiaryCode
        );
        
        /* fetchAssoc retorna false quando nada é encontrado */
        $return = $this->connection->fetchAssoc(Query::GET_FILIAL, $params);
        if(!$return) {
            return false;
        } else {
            /* não é intenção retornar os dados da filial, apenas indicar que existe */
            return true;
        }
    }
    
    /* Verifica se uma loja existe para determinada filial */
    public function storeExists($subsidiaryCode, $storeCode) {
        $params = array(
            ':CDFILIAL' => $subsidiaryCode,
            ':CDLOJA'   => $storeCode
        );
        
        /* fetchAssoc retorna false quando nada é encontrado */
        $return = $this->connection->fetchAssoc(Query::GET_LOJA, $params);
        if(!$return) {
            return false;
        } else {
            /* não é intenção retornar os dados da loja, apenas indicar que existe */
            return true;
        }
    }
    
    public function posExists($subsidiaryCode, $storeCode, $posCode) {
        $params = array(
            ':CDFILIAL' => $subsidiaryCode,
            ':CDLOJA'   => $storeCode,
            ':CDCAIXA'  => $posCode
        );
        
        $return = $this->connection->fetchAssoc(Query::GET_CAIXA, $params);
        if(!$return) {
            return false;
        } else {
            return true;
        }
    }
    
    public function consumerExists($clientCode, $consumerCode) {
        $params = array(
            ':CDCLIENTE'    => $clientCode,
            ':CDCONSUMIDOR' => $consumerCode
        );
        
        $exists = $this->connection->fetchAssoc(Query::GET_CONSUMER, $params);
        return !empty($exists);
    }


    public function getCityCode($cityCodeExt, $stateCode){
        $params = array(
            ':CDMUNICIBGE' => $cityCodeExt,
            ':SGESTADO'    => $stateCode
        );

        /* fetchAssoc retorna false quando nada é encontrado */
        $return = $this->connection->fetchAssoc(Query::GET_CITY_CODE, $params);
        if(!$return) {
            return null;
        } else {
            return $return['CDMUNICIPIO'];
        }

    }

    public function getIdcontroprod($storeCode, $subsidiaryCode){
        $params = array(
            ':CDFILIAL' => $subsidiaryCode,
            ':CDLOJA' => $storeCode
        );
        $result = $this->connection->fetchAll(Query::GET_IDCONTROPROD, $params);
        $idcontroprod = null;
        if(isset($result[0])){
            $idcontroprod = $result[0]['IDCONTROPROD'];
        }
        return $idcontroprod;
    }

    public function getMoney(){
        $params = array();
        $result = $this->connection->fetchAssoc(\Util\Query::GET_MONEY, $params);
        return $result['CDTIPORECE'];
    }  

    public function getTheThing($subsidiaryCode, $storeCode, $productCode, $idcontroprod){
        $params = array(
            ':CDFILIAL' => $subsidiaryCode,
            ':CDLOJA' => $storeCode,
            ':CDPRODUTO' => $productCode
        );
        $result = $this->connection->fetchAll(Query::GET_THE_THING, $params);
        if(isset($result[0])){
            $theThing = $result[0];
        }else{
            $theThing = null;
        }
        if ($idcontroprod === 'S' && $theThing !== null) $finalThing = '1';
        else if ($idcontroprod === 'N') $finalThing = '4';
        else $finalThing = $idcontroprod;
        return $finalThing;
    }

    public function getObservationAsProduct($obsGroupCode, $obsCode){
        $params = array(
            'CDGRPOCOR' => $obsGroupCode,
            'CDOCORR'   => $obsCode        
        );

        $result = $this->connection->fetchAssoc(Query::GET_OBSERVATION_AS_PRODUCT, $params);
        
        if($result){
            return $result['CDPRODUTO'];
        }else{
            return null;
        }
    }

    public function getObservationInfo($obsGroupCode, $obsCode){
        $params = array(
            'CDGRPOCOR' => $obsGroupCode,
            'CDOCORR'   => $obsCode        
        );
        $result = $this->connection->fetchAssoc(Query::GET_OBSERVATION_INFO, $params);
        
        return $result;
    }

    public function getBillExtId($nrcomandaven,$nrvendarest, $cdfilial){
        $params = array(
            'NRVENDAREST'   => $nrvendarest,
            'CDFILIAL'      => $cdfilial,
            'NRCOMANDA'  => $nrcomandaven       
        );

        $result = $this->connection->fetchAssoc(Query::GET_BILL_EXT_ID, $params);
        
        if($result){
            return $result['NRCOMANDAEXT'];
        }else{
            return null;
        }
    }

    public function productIsAvailable($cdfilial, $cdloja, $cdproduto){
        $params = array(
            'CDFILIAL' => $cdfilial,
            'CDLOJA'   => $cdloja,
            'CDPRODUTO' => $cdproduto 
        );

        $result = $this->connection->fetchAssoc(Query::GET_PRODUCT_IS_AVAILABLE, $params);
        
        return $result == false;

    }


    public function getSubsidiaryAddress($cdfilial, $idtpendefili){
        $params = array(
            'CDFILIAL'     => $cdfilial,
            'IDTPENDEFILI' => $idtpendefili 
        );

        $result = $this->connection->fetchAssoc(Query::GET_SUBSIDIARY_ADDRESS, $params);  
        
        if($result){
            return $result;
        }else{
            return null;
        }
        
    }
    
    public function getSubsidiaries(){
        $filiais = $this->connection->fetchAll(Query::GET_FILIAIS);
        $filiaisFinal = array();
        foreach($filiais as $filial)
            array_push($filiaisFinal, $filial['CDFILIAL']);
        return $filiaisFinal;
    }


    public function getTableStatus($cdfilial, $cdloja, $nrmesa){
        $params = array(
            'CDFILIAL' => $cdfilial,
            'CDLOJA'   => $cdloja,
            'NRMESA'   => $nrmesa 
        );

        $result = $this->connection->fetchAssoc(Query::GET_TABLE_STATUS, $params);
        if($result){
            return $result['IDSTMESAAUX'];
        }else{
            return null;
        }
    }

    public function getStoreServiceTax($cdfilial, $cdloja){
        $params = array(
            'CDFILIAL' => $cdfilial,
            'CDLOJA'   => $cdloja
        );

        $result = $this->connection->fetchAssoc(Query::GET_STORE_TAX, $params);
        if($result){
            return $result;
        }else{
            return array();
        }
    }

     public function getPositionStatus($CDFILIAL,$NRVENDAREST, $NRCOMANDA, $NRLUGARMESA){
        $params = array(
            'CDFILIAL'    => $CDFILIAL,
            'NRVENDAREST' => $NRVENDAREST,
            'NRCOMANDA'   => $NRCOMANDA,
            'NRLUGARMESA' => $NRLUGARMESA 
        );

        $result = $this->connection->fetchAssoc(Query::GET_POSITION_STATUS, $params);
        if($result){
            return $result['IDSTLUGARMESA'];
        }else{
            return null;
        }
    }

    public function updateBillServiceTax($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $IDUTILTXSERV, $VRCOMISVENDE){
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'NRVENDAREST' => $NRVENDAREST,
            'NRCOMANDA' => $NRCOMANDA,
            'IDUTILTXSERV' => $IDUTILTXSERV,
            'VRCOMISVENDE' => $VRCOMISVENDE
        );

        $this->connection->executeQuery(Query::UPDATE_BILL_SERVICE_TAX, $params);

    }

    public function getLastPosition($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $NRLUGARMESA){
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'NRVENDAREST' => $NRVENDAREST,
            'NRCOMANDA' => $NRCOMANDA
        );

        $openPositions = $this->connection->fetchAll(Query::GET_OPEN_POSITIONS, $params); 
        if(count($openPositions) == 0){
            throw new \Exception('Nenhuma posição se encontra aberta');
        }else if(count($openPositions) == 1){
            $openPosition = $openPositions[0];
            if($NRLUGARMESA == $openPosition['NRLUGARMESA']){
                return true;
            }else{
                throw new \Exception('Essa posição não se encontra aberta');
            }
        }else{
            return false;
        }
    }

    public function getDeliveryFee($CDAREA, $CDFILIAL){
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDAREA'   => $CDAREA
        );

        $deliveryFeeResult = $this->connection->fetchAssoc(Query::GET_DELIVERY_FEE, $params);
        $deliveryFee       = 0;
        if($deliveryFeeResult){
            $deliveryFee   = $deliveryFeeResult['VRTXENTRFILIAR'];
        }
        return $deliveryFee;
    }

    public function getStoreDeliveryTime($CDLOJA, $CDFILIAL){
        $params = array(
            'CDLOJA'   => $CDLOJA,
            'CDFILIAL' => $CDFILIAL 
        );

        $deliveryTimeResult = $this->connection->fetchAssoc(Query::GET_DELIVERY_TIME, $params);
        $deliveryTime = 0;
        if($deliveryTimeResult){
            $deliveryTime = $deliveryTimeResult['VRTEMPOENTREGA'];
        }
        return $deliveryTime;
    }

    public function getStoreDetails($CDFILIAL,$CDLOJA){
        $params = array(
            'CDLOJA'   => $CDLOJA,
            'CDFILIAL' => $CDFILIAL 
        );

        return $this->connection->fetchAssoc(Query::GET_STORE_DETAILS, $params);
    }
    
    public function getBaseUrl($subsidiaryCode){
        $params = array(
            'CDFILIAL' => $subsidiaryCode    
        );
        if($this->utilOdhen->databaseIsOracle()){
            $baseUrl = $this->connection->fetchAssoc(Query::GET_BASEURL, $params);
        }else{
            $baseUrl = $this->connection->fetchAssoc(Query::SQL_GET_BASEURL, $params);
        }
        if(empty($baseUrl)) throw new \Exception("Extratcons URL não encontrada.");
        return $baseUrl['CDURLWSEXTCONS'];
    }

    public function getBalanceValue($CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD){
        $params = array(
            'CDCLIENTE'    => $CDCLIENTE,
            'CDCONSUMIDOR' => $CDCONSUMIDOR,
            'CDFAMILISALD' => $CDFAMILISALD
        );
        $balanceValueResult = $this->connection->fetchAssoc(Query::GET_BALANCE_VALUE, $params);
        $balanceValue = null;
        if($balanceValueResult){
            $balanceValue = $balanceValueResult['VRSALDCONFAM'];
        }
        return $balanceValue;
    }
    public function getConsumerBalanceInfo($CDCLIENTE, $CDCONSUMIDOR){
        $params = array(
            'CDCLIENTE'    => $CDCLIENTE,
            'CDCONSUMIDOR' => $CDCONSUMIDOR
        );
        return $this->connection->fetchAssoc(Query::GET_CONSUMER_LIMIT, $params);
    }

    public function getPositionItems($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $positionNumber){
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'NRVENDAREST' => $NRVENDAREST,
            'NRCOMANDA' => $NRCOMANDA,
            'NRLUGARMESA' => $positionNumber
        );

        $positionItems = $this->connection->fetchAll(Query::GET_POSITION_ITEMS, $params);

        return $positionItems;
    }
    
    public function getDefaultClient($CDFILIAL){
        $params = array(
            'CDFILIAL' => $CDFILIAL    
        );
        
        $client = $this->connection->fetchAssoc(Query::GET_DEFAULT_CONSUMER,$params);
        
        if($client)
            return $client['CDCLIENTE'];
        else
            throw new \Exception('Cliente padrão inexistente.');
        
    }
    
    public function getToTravelObservation(){
        return $this->connection->fetchAssoc(Query::GET_TOTRAVEL_OBSERVATION);
    }
    public function getDriveThruObservation(){
        return $this->connection->fetchAssoc(Query::GET_DRIVETHRU_OBSERVATION);
    }
    public function getTotravelObservationOrder($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $NRSEQVENDA, $CDGRPOCOR, $CDOCORR){
        if(!empty($NRCOMANDA)){
            $comandaParams = array(
                "NRVENDAREST" => $NRVENDAREST,
                "NRCOMANDA"   => $NRCOMANDA
            );
            $query = Query::GET_TOTRAVELOBS_ORDER;
        }else{
            $comandaParams = array(
                "NRSEQVENDA"   => $NRSEQVENDA
            );
            $query = Query::GET_TOTRAVELOBS_SALE;
        }
        $params = array(
            "CDFILIAL"      => $CDFILIAL,
            "CDGRPOCOR"     => $CDGRPOCOR,
            "CDOCORR"       => $CDOCORR
        );
        $params = array_merge($params, $comandaParams);
        $return = $this->connection->fetchAssoc($query, $params);
        if(!empty($return)){
            return true;
        }
        return false;
    }
    
    public function getTablePositions($CDFILIAL , $NRVENDAREST){
        $params = array(
            'CDFILIAL'    => $CDFILIAL,
            'NRVENDAREST' => $NRVENDAREST
        );
        $positions = $this->connection->fetchAll(Query::GET_TABLE_POSITIONS,$params);
        
        $retorno = array();   
        if(count($positions) > 0){
            foreach($positions as $posicao){
                array_push($retorno,$posicao['NRLUGARMESA']);
            }
            return $retorno;
        }
            
        else 
            throw new \Exception('Esta mesa não está aberta no momento.');
        
    }

    public function updateVendaRestConsumer($CDFILIAL, $NRVENDAREST, $CDCLIENTE, $CDCONSUMIDOR){
        $params = array(
            'CDFILIAL'      => $CDFILIAL,
            'NRVENDAREST'   => $NRVENDAREST,
            'CDCLIENTE'     => $CDCLIENTE,
            'CDCONSUMIDOR'  => $CDCONSUMIDOR
        );

        $this->connection->executeQuery(Query::UPDATE_VENDAREST_CONSUMER,$params);
    }

    public function saveLog($inputJSON, $folder, $fileName) {        
	    $basePath = $this->logPath;
	    
        if(!file_exists($basePath.DIRECTORY_SEPARATOR.$folder)) 
            $this->createFolder($basePath, $folder);
        
        $myfile = fopen($basePath.DIRECTORY_SEPARATOR.$folder.DIRECTORY_SEPARATOR.$fileName.date("d-m-Y").".txt", "a+");

        $txt = date('H:i:s')." - ".$inputJSON."\r\n";
        fwrite($myfile, $txt);
        fclose($myfile);
    }

    public function saveLogOrder($inputJSON, $folder, $fileName) {
	    $basePath = $this->logPath;
        
        if(!file_exists($basePath.DIRECTORY_SEPARATOR.$folder)) 
            $this->createFolder($basePath, $folder);

        $myfile = fopen($basePath.DIRECTORY_SEPARATOR.$folder.DIRECTORY_SEPARATOR.$fileName.".txt", "w");

        $txt = $inputJSON;
        fwrite($myfile, $txt);
        fclose($myfile);
      }

    private function createFolder($basePath, $folder) {
        if($this->isRunningOnWindows()) {
            mkdir($basePath.DIRECTORY_SEPARATOR.$folder, 0777, true);
        } else {
            mkdir($basePath.DIRECTORY_SEPARATOR.$folder, 0775, true);
        }
    }

    public function getOrderStatus($CDFILIAL, $NRCOMANDA, $NRCOMANDAEXT){
        $params = array(
            'CDFILIAL'      => $CDFILIAL,
            'NRCOMANDA'     => $NRCOMANDA,
            'NRCOMANDAEXT'  => $NRCOMANDAEXT
        );

        return $this->connection->fetchAssoc(Query::GET_ORDER_STATUS, $params);
    }
    
    public function setOrderArrival($subsidiaryCode, $saleCode, $saleId, $arrivalTime){
        $params = array(
            'CDFILIAL'     => $subsidiaryCode,  
            'NRCOMANDA'    => $saleCode,
            'NRCOMANDAEXT' => $saleId, 
            'DTCHEGACMD'   => $arrivalTime    
        );
        return $this->connection->executeUpdate(Query::SET_ORDER_ARRIVAL, $params);
        
    }
    
    public function getMinTime($subsidiaryCode, $storeCode){
        $params = array(
            'CDFILIAL' => $subsidiaryCode,
            'CDLOJA'   => $storeCode
        );
        
        $time = $this->connection->fetchAssoc(Query::GET_MIN_TIME, $params);
        
        return $time['VRTEMPOMIN'];   
    }
    
    public function getNrvendarest($cdfilial, $nrcomanda){
        $params = array(
            'CDFILIAL'  => $cdfilial,
            'NRCOMANDA' => $nrcomanda
        );
        return $this->connection->fetchAssoc(Query::GET_NRVENDAREST, $params);
    }
    
    public function cancelProducts($cdfilial,$nrvendarest,$nrcomanda){
        $params = array(
            'CDFILIAL'    => $cdfilial,   
            'NRVENDAREST' => $nrvendarest,   
            'NRCOMANDA'   => $nrcomanda    
        );
        $this->connection->executeQuery(Query::CANCEL_PRODUCTS, $params);
        return $this->connection->fetchAll(Query::GET_BILL_LAST_ITEM,$params);
    }
    
    public function insertKDSTEMP($kdsSeq,$dsOperacao){
        $params = array(
            'NRSEQOPERACAO' => $kdsSeq,
            'DSOPERACAO' => json_encode($dsOperacao,true)
        );
        $this->connection->executeQuery(Query::INSERT_KDSOPERACAOTEMP, $params);
    }
    
    public function validatePaymentCode($paymentCode){
        $params = array(
            'CDTIPORECE' => $paymentCode
        );
        $payment = $this->connection->fetchAssoc(Query::VALIDATE_PAYMENT, $params);
        if(empty($payment)) throw new \Exception("Forma de pagamento não encontrada.");
        return true;
        
    }
    
    public function updateConsumerPhone($clientCode, $consumerCode, $phoneToUpdate){
        $params = array(
            'CDCLIENTE'    => $clientCode,
            'CDCONSUMIDOR' => $consumerCode,
            'NRTELECONS'   => $phoneToUpdate
        );
        
        $this->connection->executeQuery(Query::UPDATE_CONSUMER_PHONE, $params);
    }
    
    public function getTipoCons($clientCode, $consumerCode){
        $params = array(
            'CDCLIENTE' => $clientCode,
            'CDCONSUMIDOR' => $consumerCode
        );   
        $consumerType = $this->connection->fetchAssoc(Query::GET_CONSUMERTYPE, $params);
        return $consumerType['CDTIPOCONS'];
    }
    
    public function getCreditPayment(){
        $paymentCode = $this->connection->fetchAssoc(Query::GET_CREDITPAYMENT);
        return $paymentCode['CDTIPORECE'];
    }
    
    public function getSaleStatus($cdfilial, $cdcaixa, $nrseqvenda){
        $params = array(
            'CDFILIAL'    => $cdfilial,
            'CDCAIXA'     => $cdcaixa,
            'NRSEQVENDA'  => $nrseqvenda
        );
        
        $data = $this->connection->fetchAssoc(Query::GET_SALE_STATUS, $params);
        return $data;
    }
    
    public function cancelComandaven($cdfilial, $nrvendarest, $nrcomanda) {
        $params = array(
            'IDSTCOMANDA' => '4',
            'CDFILIAL'    => $cdfilial,
            'NRVENDAREST' => $nrvendarest,
            'NRCOMANDA'   => $nrcomanda
        );

        $this->connection->executeQuery(Query::CANCEL_COMANDAVEN, $params);
    }

    public function deleteItcomandaven($nrvendarest, $cdfilial, $nrcomanda){
        $params = array(
            'NRVENDAREST' => $nrvendarest,
            'CDFILIAL'    => $cdfilial,
            'NRCOMANDA'   => $nrcomanda
        );
        $this->connection->executeQuery(Query::DELETE_ITCOMANDAVEN, $params);
    }

    public function deleteMovCaixaDlv($nrvendarest, $cdfilial){
        $params = array(
            'NRVENDAREST' => $nrvendarest,
            'CDFILIAL'    => $cdfilial
        );
        $this->connection->executeQuery(Query::DELETE_MOVCAIXADLV, $params);
    }
    
    public function deleteComandaven($nrvendarest, $cdfilial, $nrcomanda){
        $params = array(
            'NRVENDAREST' => $nrvendarest,
            'CDFILIAL'    => $cdfilial,
            'NRCOMANDA'   => $nrcomanda
        );
        $this->connection->executeQuery(Query::DELETE_COMANDAVEN, $params);
    }
    
    public function deleteVendarest($nrvendarest, $cdfilial){
        $params = array(
            'NRVENDAREST' => $nrvendarest,
            'CDFILIAL'    => $cdfilial
        );
        if(!$this->databaseIsOracle()) $this->connection->executeQuery(Query::DELETE_VENDAREST, $params);
    }

    public function getValueFidelityOrders($cdcliente, $cdconsumidor){
        $params = array(
            'CDCLIENTE' => $cdcliente,
            'CDCONSUMIDOR' => $cdconsumidor
        );
        $value = $this->connection->fetchAssoc(Query::GET_VALUE_FIDELITY_ORDERS, $params);
        if(!empty($value)) return $value['VRDESCFID'];
        else return 0;
    }
    
    public function getConsumerBalance($cdcliente, $cdconsumidor, $cdfamilisald, $timeout=10, $cdfilial = null){
        $useExtCons = $this->getUseExtratocons($cdfilial);
        if($cdfilial == null || $useExtCons['IDEXTCONSONLINE'] === 'S'){
            $params = array(
                'P_CDFILIAL' => '',
                'P_CDCLIENTE' => $cdcliente,
                'P_CDCONSUMIDOR' => $cdconsumidor,
                'P_CDFAMILISALD' => $cdfamilisald,
                'P_DTEXTRATO' => '',
                'P_TPOPERACAO' => 'CONSULTA_SALDO'
            );
            $saldo = $this->callExtratocons($params, 'T', "RecuperaDados", $timeout);
            $xmlSaldo = simplexml_load_string($saldo);
            $saldo = $xmlSaldo->ExtratoCons->Registro->VRSALDCONFAM;
            if(!empty($saldo)) $saldo = $this->removeDecimal($saldo);
            else return 0;
            
            $todayPayments = $this->getTodayPayments($cdcliente, $cdconsumidor);
            if(isset($todayPayments)) $saldo -= $todayPayments;

            return $saldo;
        } else {
            $consultaSaldo = $this->consumidorAPI->consultaSaldo($cdfilial, $cdcliente, $cdconsumidor, $useExtCons['NRORG']);
            if(!empty($consultaSaldo)){
                $saldo = $this->removeDecimal(str_replace(",", ".", $consultaSaldo[0]['VRSALDCONFAM']));
            } else return 0;
            return $saldo;
        }
    }
    
    public function getTodaysExpenditure($cdcliente, $cdconsumidor, $timeout=10, $cdfilial = null){
        $useExtCons = $this->getUseExtratocons($cdfilial);
        if($cdfilial == null || $useExtCons['IDEXTCONSONLINE'] === 'S'){
            $params = array(
                'P_CDFILIAL' => '',
                'P_CDCLIENTE' => $cdcliente,
                'P_CDCONSUMIDOR' => $cdconsumidor,
                'P_CDFAMILISALD' => '',
                'P_DTEXTRATO' => date('d/m/Y'),
                'P_TPOPERACAO' => 'CONSULTA_CONSUMO'
            );
            $todayExpenditure = $this->callExtratocons($params, 'T', "RecuperaDados", $timeout);
            $xmlSaldo = simplexml_load_string($todayExpenditure);
            $todayExpenditure = $xmlSaldo->ExtratoCons->Registro->VALCONSUM;
            if(!empty($todayExpenditure)) $todayExpenditure = $this->removeDecimal($todayExpenditure);
            else $todayExpenditure = 0;
            return $todayExpenditure;
        } else {
            $consultaConsumo = $this->consumidorAPI->calculaConsumoDiario($cdfilial, $cdcliente, $cdconsumidor, $useExtCons['NRORG']);
            if(!empty($consultaConsumo)){
                $consumo = $this->removeDecimal(str_replace(",", ".", $consultaConsumo));
            } else return 0;
            return $consumo;
        }
    }

    public function getConsumerFidelity($cdcliente, $cdconsumidor){
        $cdfamilisald = '010';
        $params = array(
          'P_CDFILIAL' => '',
          'P_CDCLIENTE' => $cdcliente,
          'P_CDCONSUMIDOR' => $cdconsumidor,
          'P_CDFAMILISALD' => $cdfamilisald,
          'P_DTEXTRATO' => '',
          'P_TPOPERACAO' => 'CONSULTA_SALDO_FID'
        );
        $saldo = $this->callExtratocons($params, 'T', "RecuperaDados");
        $xmlSaldo = simplexml_load_string($saldo);
        $saldo = $xmlSaldo->ExtratoCons->Registro->VRSALDCONEXT;
        $consumerInfo = $this->connection->fetchAssoc(Query::GET_CONSUMER_FIDEL_INFO, $params);
        $consumerInfo['value'] = $this->removeDecimal($saldo);
        if(!empty($consumerInfo['usesPartialValue']))
            $consumerInfo['usesPartialValue'] = filter_var($consumerInfo['usesPartialValue'], FILTER_VALIDATE_BOOLEAN);
        else
            $consumerInfo['usesPartialValue'] = false;
        
        return $consumerInfo;
      }
    
    public function callExtratocons($params, $subsidiaryCode, $function, $timeout = 10){
        $postfields = "";
        foreach ($params as $key => $param){
            $postfields .= "&".$key."=".$param;
        }
        $postfields = substr($postfields, 1);

        $URL = $this->getBaseUrl($subsidiaryCode);
        $URL = rtrim($URL, '/') . "/";
        $curl = curl_init();
        
        curl_setopt($curl, CURLOPT_URL, $URL.$function);
        curl_setopt($curl, CURLOPT_POST, 1);
        curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type: application/x-www-form-urlencoded'));
        curl_setopt($curl, CURLOPT_POSTFIELDS, $postfields);
        curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_TIMEOUT, $timeout);
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 5);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);

        $curl_response = curl_exec($curl);
        $err    = curl_errno($curl);
        $errmsg = curl_error($curl);

        $ainfo = curl_getinfo($curl);
        if (is_array($ainfo)) {
            $this->soapCurlLog($ainfo);
        }

        curl_close($curl);

        if ($err) {
            if ($err === 28) {
                throw new \Exception("<br>Nao foi possivel estabelecer conexao com o servidor do EXTRATOCONS ONLINE.");
            } else {
                throw new \Exception("<br>Ocorreu um erro na chamada do EXTRATOCONS ONLINE:<br><br>" . $errmsg);
            }
        }

        if ($function == 'RecuperaDados') {
            if (!self::isXml($curl_response)) {
                throw new \Exception("<br>O EXTRATOCONS ONLINE retornou uma mensagem inesperada:<br><br>" . $curl_response);
            } else {
                return $curl_response;
            }
        } 

        if ($function == 'InsereDados' && $curl_response !== "OK") {
            throw new \Exception("<br>O EXTRATOCONS ONLINE retornou um erro:<br><br>" . $curl_response);
        }

        if($curl_response !== "OK")
            throw new \Exception("<br>O EXTRATOCONS ONLINE retornou um erro:<br><br>" . $curl_response);
        
    }

    public function isXml($string){
        return substr($string, 0, 5) == "<?xml";
    }
    
    public function getSalesInfo($initialDate, $finalDate, $cpf){
        $params = array(
            'INITIALDATE' => $initialDate,
            'FINALDATE' => $finalDate,
            'CPF' => $cpf
        );
        
        return $this->connection->fetchAll(Query::GET_SALES_INFO, $params);
    }
    
    public function getValuesInfo($cdfilial, $cdcaixa, $nrseqvenda){
        $params = array(
            'CDFILIAL' => $cdfilial,
            'CDCAIXA' => $cdcaixa,
            'NRSEQVENDA' => $nrseqvenda
        );
        
        return $this->connection->fetchAssoc(Query::GET_VALUES_INFO, $params);
    }

    public function newCode($contador, $size = '20'){
        try{
            $result = "";
            $stmt = $this->connection->prepare("CALL NOVO_CODIGO_NT(?, ?, ?)");
            $stmt->bindValue(1, $contador, 1);
            $stmt->bindParam(2, $result, 1, 20);
            $stmt->bindValue(3, 1, 1);
            $stmt->execute();
            return substr($result, strlen($result) - $size);
        }catch(\Exception $e){
            throw new \Exception("Falha ao executar procedure NOVO_CODIGO");
        }
    }

    public function newCodeWt($counterName, $size = '20'){
        $nrseq = null;
        $storedProcedure = new StoredProcedure($this->connection, 'NOVO_CODIGO');

        $storedProcedure->addParam(new Param('P_CONTADOR', Param::PARAM_INPUT, $counterName, Param::PARAM_TYPE_STR, strlen($counterName)));
        $storedProcedure->addParam(new Param('P_SEQUENCIAL', Param::PARAM_OUTPUT, $nrseq, Param::PARAM_TYPE_STR, 20));
        $storedProcedure->execute();
    }
    
    public function getNewCode($CDCONTADOR, $size = 20){
        $params = array(
            ':CDCONTADOR' => $CDCONTADOR
        );
        $code = $this->connection->fetchAssoc("SQL_NOVO_CODIGO", $params);
        $code = substr($code['NRSEQUENCIAL'], strlen($code['NRSEQUENCIAL']) - $size);
        return $code;
    }

    public function getComandaInfo($nrcomanda, $nrmesa){
        $params = array(
            'NRCOMANDA' => $nrcomanda,
            'NRMESA' => $nrmesa
        );

        $comandaInfo = $this->connection->fetchAssoc(Query::GET_COMANDA_INFO, $params);
        if(empty($comandaInfo)) throw new \Exception("Pedido não encontrado");
        return $comandaInfo;
    }

    public function getDadosImpressora(){
        return $this->connection->fetchAssoc(Query::GET_DADOS_IMPRESSORA);
    }

    public function getTempoPermanencia($cdfilial, $nrvendarest, $nrcomanda) {
        $params = array(
            'CDFILIAL' => $cdfilial,
            'NRVENDAREST' => $nrvendarest,
            'NRCOMANDA' => $nrcomanda
        );

        return $this->connection->fetchAssoc(Query::SQL_TEMPO_PERMANENCIA, $params);
    }

    public function getProdutosParcial($cdfilial, $stComandaVens, $stPosicoes, $impPosicao){
        $params = array(
            'CDFILIAL' => $cdfilial,
            'STCOMANDAVENS' => $stComandaVens,
            'STPOSICOES' => $stPosicoes,
            'IMPPOSICAO' => $impPosicao
        );

        return $this->connection->fetchAll(Query::SQL_PRODUTOS_PARCIAL, $params);
    }

    public function getComissaoVenda($cdfilial, $nrvendarest, $nrcomanda){
        $params = array(
			'CDFILIAL' => $cdfilial,
			'NRVENDAREST' => $nrvendarest,
			'NRCOMANDA' => $nrcomanda
		);

        return $this->connection->fetchAssoc(Query::SQL_GET_COMISSAO_VENDA, $params);
    }

    public function getImprLoja($cdfilial, $cdloja, $nrseqimprloja){
        $params = array(
			'CDFILIAL' => $cdfilial,
			'CDLOJA' => $cdloja,
			'NRSEQIMPRLOJA' => $nrseqimprloja
		);
        
        return $this->connection->fetchAssoc(Query::SQL_VAL_IMPRE_LOJA, $params);
    }

    public function getPosicaoParcial($cdfilial, $nrvendarest, $nrcomanda){
        $params = array(
			'CDFILIAL' => $cdfilial,
			'NRVENDAREST' => $nrvendarest,
			'NRCOMANDA' => $nrcomanda
		);
        
        return $this->connection->fetchAll(Query::SQL_POSICAO_PARCIAL, $params);
    }

    public function getProdutosParcialCouver($cdfilial, $nrvendarest, $nrcomanda, $cdproduto){
        $params = array(
			'CDFILIAL' => $cdfilial,
			'NRVENDAREST' => $nrvendarest,
            'NRCOMANDA' => $nrcomanda,
            'CDPRODUTO' => $cdproduto
		);
        
        return $this->connection->fetchAssoc(Query::SQL_PRODUTOS_PARCIAL_CDPRODCOUVER, $params);
    }

    public function getSonDiscount($sonProductCode, $itemProductCode, $CDGRUPROMOC){
        $params = array(
            'CDPRODUTO' => $sonProductCode,
            'CDPRODPROMOCAO' => $itemProductCode,
            'CDGRUPROMOC' => $CDGRUPROMOC
        );
        return $this->connection->fetchAssoc(Query::GET_SON_COMBO, $params);
    }

    public function removeAcentos($texto) {
        $table = array('á' => 'a', 'à' => 'a', 'ã' => 'a', 'â' => 'a',
            'é' => 'e', 'ê' => 'e', 'í' => 'i', 'ó' => 'o', 'ô' => 'o',
            'õ' => 'o', 'ú' => 'u', 'ü' => 'u', 'ç' => 'c', 'Á' => 'A',
            'À' => 'A', 'Ã' => 'A', 'Â' => 'A', 'É' => 'E', 'Ê' => 'E',
            'Í' => 'I', 'Ó' => 'O', 'Ô' => 'O', 'Õ' => 'O', 'Ú' => 'U',
            'Ü' => 'U', 'Ç' => 'C_');
        return preg_replace("[^a-zA-Z0-9_]", "", strtr($texto, $table));
    }

    public function formataPreco($valor){
        return number_format($this->truncate($valor, 2), 2, ",", "");
    }

    public function toRight($leftText, $rightText, $lineWidth){
        $spaceToFill = $lineWidth - (strlen($leftText) + strlen($rightText));
        $result = $leftText . self::replicate(' ', $spaceToFill);
        $result .= $rightText;
        return $result;
    }

    public function truncate($val, $f="0") {
        if(($p = strpos($val, '.')) !== false) {
            $val = floatval(substr($val, 0, $p + 1 + $f));
        }
        return $val;
    }

    public function replicate($char, $numero){
        $result = $char;
        for ($i = 1; $i < $numero; ++$i){
            $result .= $char;
        }
        return $result;
    }

    public function getCDCAIXA($CDFILIAL, $NRCOMANDA, $NRVENDAREST){
        $params = array(
            'CDFILIAL'    => $CDFILIAL,
            'NRCOMANDA'   => $NRCOMANDA,
            'NRVENDAREST' => $NRVENDAREST
        );
        return $this->connection->fetchAssoc(Query::GET_CDCAIXA_ITCOMANDAVEN, $params);
    }

    public function recuperaCaixaRecebedor() {
        if(!$this->databaseIsOracle()) $caixa = $this->connection->fetchAssoc(Query::GET_CAIXA_RECEBEDOR);
        else $caixa = $this->connection->fetchAssoc(Query::GET_CAIXA_RECEBEDOR_ORACLE);
        return $caixa['CDCAIXA'];
    }

    public function getSaleXml($cdfilial, $cdcaixa, $nrcomanda){
        $params = array(
            'CDFILIAL' => $cdfilial,
            'CDCAIXA' => $cdcaixa,
            'NRCOMANDA' => $nrcomanda
        );

        $xml = $this->connection->fetchAssoc(Query::GET_SALE_XML, $params);
        return $xml['DSARQXMLNFCE'];
    }
    
    public function getSaleDate($cdfilial, $cdcaixa, $nrcomanda){
        $params = array(
            'CDFILIAL' => $cdfilial,
            'CDCAIXA' => $cdcaixa,
            'NRCOMANDA' => $nrcomanda
        );

        $xml = $this->connection->fetchAssoc(Query::GET_SALE_DATE, $params);
        return $xml['DTENTRVENDA'];
    }

    public function testConnection(){
        return $this->connection->connect();
    }
    
    public function getStoreName($cdfilial, $cdcaixa, $cdloja){
        $params = array(
            'CDFILIAL' => $cdfilial,
            'CDCAIXA' => $cdcaixa,
            'CDLOJA' => $cdloja
        );
        return $this->connection->fetchAssoc(Query::GET_STORE_NAME, $params);
    }
    
    public function getSaleDetails($cdfilial, $nrcomanda, $nrvendarest, $systemPath, $cdcaixa){
        $params = array(
            'CDFILIAL' => $cdfilial,
            'NRCOMANDA' => $nrcomanda,
            'NRVENDAREST' => $nrvendarest
        );
        
        if(!$this->databaseIsOracle()) $saleDetail = $this->connection->fetchAssoc(Query::GET_SALE_DETAILS_SQL, $params);
        else $saleDetail = $this->connection->fetchAssoc(Query::GET_SALE_DETAILS_ORACLE, $params);
        /*if(empty($saleDetail['DSARQXMLNFCE'])){
            $IDAMBTRABNFCE = $this->connection->fetchAssoc(Query::GET_IDAMBTRABNFCE, $params)['IDAMBTRABNFCE'];
            $typePath = $IDAMBTRABNFCE == '2' ? 'homologacao' : 'producao';
            $XMLPath = realpath($this->systemPath) . DIRECTORY_SEPARATOR . 'NFCE' . DIRECTORY_SEPARATOR . 'NOTAS' . DIRECTORY_SEPARATOR . $typePath . DIRECTORY_SEPARATOR."ASSINADAS". DIRECTORY_SEPARATOR;
            $today = date('Y-m-d');
            $nfceMoment = "1-$cdfilial-$cdcaixa"."_$today->".DIRECTORY_SEPARATOR;
            $nracessonfce = $saleDetail['NRACESSONFCE'];
            if(is_dir($XMLPath.$nfceMoment)){
                if(file_exists($XMLPath.$nfceMoment.$nracessonfce."xml")){
                    $saleDetail['DSARQXMLNFCE'] = file_get_contents($XMLPath.$nfceMoment.$nracessonfce."xml");
                }
            }
        }*/
        return $saleDetail;
    }
    
    public function getTodayPayments($clientCode, $consumerCode){
        $params = array(
            'CDCLIENTE' => $clientCode,
            'CDCONSUMIDOR' => $consumerCode
        );
        if($this->databaseIsOracle()) $result = $this->connection->fetchAssoc(Query::GET_PERSONAL_PAYMENTS_ORACLE, $params)['VRMOVIVENDDLV'];
        else $result = $this->connection->fetchAssoc(Query::GET_PERSONAL_PAYMENTS_SQL, $params)['VRMOVIVENDDLV'];
        return $result;
    }
    
    public function databaseIsOracle(){
        return $this->utilOdhen->databaseIsOracle();
    }

    public function convertToHoursMins($time, $format = '%02d:%02d') {
        if ($time < 1) {
            return;
        }
        $hours = floor($time / 60);
        $minutes = ($time % 60);
        return sprintf($format, $hours, $minutes);
    }

    public function getPosOperator($cdfilial, $cdcaixa) {
      $params = array(
        ':CDFILIAL' => $cdfilial,
        ':CDCAIXA' => $cdcaixa
      );
      return $this->connection->fetchAssoc(\Util\Query::GET_POS_OPERATOR, $params);
  	}

    public function insertLogOperFos($cdfilial, $cdcaixa, $idoperfos, $dslivrefos) {
        $oper = $this->getPosOperator($cdfilial, $cdcaixa);

        $params = array(
			      ':CDFILIAL'     => $cdfilial,
            ':CDCAIXA'      => $cdcaixa,
            ':IDOPERFOS'    => $idoperfos,
            ':CDOPERCAIXA'  => $oper['CDOPERADOR'],
            ':NMOPERCAIXA'  => $oper['NMOPERADOR'],
            ':CDOPERRESPOP' => $oper['CDOPERADOR'],
            ':NMOPERRESPOP' => $oper['NMOPERADOR'],
            ':DSLIVREFOS'   => $dslivrefos
        );
        
        if($this->databaseIsOracle()) $this->connection->executeQuery(Query::INSERT_LOG, $params);
        else $this->connection->executeQuery(Query::SQL_INSERT_LOG, $params);
    }

    public function getSaleByOrderCode($cdfilial, $cdcaixa, $nrcomanda){
        $params = array(
            'CDFILIAL'    => $cdfilial,
            'CDCAIXA'     => $cdcaixa,
            'NRCOMANDAVND'   => $nrcomanda
        );
        
        $data = $this->connection->fetchAssoc(Query::GET_SALE_ORDER_ID, $params);
        return $data;
    }

    private function soapCurlLog($ainfo){
        try{
            $path = __DIR__."/../../../log/ExtratoCons/".date("Y/m/d/H")."/";
        
            if(!is_dir($path)) {
                if($this->isRunningOnWindows()) {
                    mkdir($path, 0777, true);
                } else {
                    mkdir($path, 0775, true);
                }
            }
            
            file_put_contents($path.round(microtime(true) * 1000).".json", json_encode($ainfo, JSON_PRETTY_PRINT));
        } catch(\Exception $e){
            
        }
    }

    public function updateOrderClosingType($cdfilial, $nrcomanda, $nrvendarest) {
        $params = array(
            ':CDFILIAL' => $cdfilial,
            ':NRCOMANDA' => $nrcomanda,
            ':NRVENDAREST' => $nrvendarest
        );
        $this->connection->executeQuery(Query::UPDATE_ORDER_CLOSING_TYPE, $params);
    }
    
    public function isRunningOnWindows() {
        return defined('PHP_WINDOWS_VERSION_BUILD');
    }

    public function getFidelityUtil($cdfilial){
        $params = array(
            ':CDFILIAL' => $cdfilial
        );
        return $this->connection()->fetchAssoc(Query::GET_FID_UDS, $params);
    }

    public function getUseExtratocons($cdfilial) {
        $params = array(
            ':CDFILIAL' => $cdfilial
        );
        return $this->connection->fetchAssoc(Query::GET_USE_EXT_CONS, $params);
    }

    public function getEnvironmentParam($param) {
        return $this->instanceManager->getParameter($param);
    }

    public function buildCachedImages($fileName) {
        $separator = DIRECTORY_SEPARATOR;
        $basePath  = preg_replace('/[\/\\\\]/', $separator, $this->getEnvironmentParam('IMAGES_PATH'));

        $largeFile = (file_exists("{$basePath}{$separator}L-{$fileName}") ? "IMAGES/L-{$fileName}" : null);
        $smallFile = (file_exists("{$basePath}{$separator}S-{$fileName}") ? "IMAGES/S-{$fileName}" : null);

        return [
            'large' => $largeFile,
            'small' => $smallFile
        ];        
    }

    public function getLimitDebConsumer($cdfilial, $cdloja, $cdcliente, $cdconsumidor){
        return $this->consumidorAPI->getLimitDebConsumer($cdfilial, $cdloja, $cdcliente, $cdconsumidor);
    }

    public function getLimiteCredDisponivel($cdfilial, $cdcliente, $cdconsumidor){
        return $this->consumidorAPI->getLimiteCredDisponivel($cdfilial, $cdcliente, $cdconsumidor);
    }

    public function calculaConsumoDiario($cdfilial, $cdcliente, $cdconsumidor, $cdfamilisald){
        return $this->consumidorAPI->calculaConsumoDiario($cdfilial, $cdcliente, $cdconsumidor, $cdfamilisald);
    }

    public function calculaConsumoMensal($cdfilial, $cdcliente, $cdconsumidor, $cdfamilisald){
        return $this->consumidorAPI->calculaConsumoMensal($cdfilial, $cdcliente, $cdconsumidor, $cdfamilisald);
    }

    public function buscaSaldoExtrato($cdcliente, $cdconsumidor, $cdfamilisald, $cdfilial){
        return $this->consumidorAPI->buscaSaldoExtrato($cdcliente, $cdconsumidor, $cdfamilisald, $cdfilial);
    }

    public function getSaldoDebitoConsumidor($cdcliente, $cdconsumidor){
        return $this->consumidorAPI->getSaldoConsumidor($cdcliente, $cdconsumidor);
    }

    public function isValidCPF($cpf) {
      $cpf = preg_replace( '/[^0-9]/is', '', $cpf );
       
      if (strlen($cpf) == 11) {
          
           // Extrai somente os números
           $cpf = preg_replace( '/[^0-9]/is', '', $cpf );
            
           // Verifica se foi informado todos os digitos corretamente
           if (strlen($cpf) != 11) {
               return false;
           }
       
           // Verifica se foi informada uma sequência de digitos repetidos. Ex: 111.111.111-11
           if (preg_match('/(\d)\1{10}/', $cpf)) {
               return false;
           }
       
           // Faz o calculo para validar o CPF
           for ($t = 9; $t < 11; $t++) {
               for ($d = 0, $c = 0; $c < $t; $c++) {
                   $d += $cpf[$c] * (($t + 1) - $c);
               }
               $d = ((10 * $d) % 11) % 10;
               if ($cpf[$c] != $d) {
                   return false;
               }
           }
              
      } else if (strlen($cpf) == 14) {
      
            $cpf = preg_replace('/[^0-9]/', '', (string) $cpf);
        
            // Valida tamanho
            if (strlen($cpf) != 14)
              return false;
      
            // Verifica se todos os digitos são iguais
            if (preg_match('/(\d)\1{13}/', $cpf))
              return false;	
          
            // Valida primeiro dígito verificador
            for ($i = 0, $j = 5, $soma = 0; $i < 12; $i++)
                {
                  $soma += $cpf[$i] * $j;
                  $j = ($j == 2) ? 9 : $j - 1;
                }          
            $resto = $soma % 11;          
            if ($cpf[12] != ($resto < 2 ? 0 : 11 - $resto))
              return false;
          
            // Valida segundo dígito verificador
            for ($i = 0, $j = 6, $soma = 0; $i < 13; $i++)
                {
                      $soma += $cpf[$i] * $j;
                      $j = ($j == 2) ? 9 : $j - 1;
                }          
            $resto = $soma % 11;          
            return $cpf[13] == ($resto < 2 ? 0 : 11 - $resto);
      } else {
           return false;
      }      
      return true;
   }
}